package com.yunzhi.demo.shiro.realms.manager;

import com.yunzhi.demo.shiro.utils.JWTToken;
import com.yunzhi.demo.shiro.utils.JWTUtil;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;

import java.util.Set;

public class ManagerRealm extends AuthorizingRealm {

    IManagerService iManagerService;

    public void setManagerService(IManagerService iManagerService) {
        this.iManagerService = iManagerService;
    }

    @Override
    public boolean supports(AuthenticationToken token) {
        return token instanceof JWTToken;
    }

    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        // 获取用户角色，权限
        String token = (String) principalCollection.getPrimaryPrincipal();
        String loginId = JWTUtil.getLoginId(token);
        Set<String> roles = iManagerService.getRolesByLoginId(loginId);
        Set<String> permissions = iManagerService.getPermissionsLoginId(loginId);

        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
        simpleAuthorizationInfo.setStringPermissions(permissions);
        simpleAuthorizationInfo.setRoles(roles);
        return simpleAuthorizationInfo;
    }

    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        String token = (String) authenticationToken.getPrincipal();
        String loginId = JWTUtil.getLoginId(token);

        if (!iManagerService.verify(loginId)) {
            throw new AuthenticationException("User is abnormal");
        }

        // 交给 AuthenticatingRealm 使用 CredentialsMatcher 行校验
        return new SimpleAuthenticationInfo(token, token, getName());
    }
}
